// 
// File:          Inherit_H_Impl.cxx
// Symbol:        Inherit.H-v1.1
// Symbol Type:   class
// Babel Version: 1.4.0 (Revision: 6607 release-1-4-0-branch)
// Description:   Server-side implementation for Inherit.H
// 
// WARNING: Automatically generated; only changes within splicers preserved
// 
// 
#include "Inherit_H_Impl.hxx"

// 
// Includes for all method dependencies.
// 
#ifndef included_sidl_BaseInterface_hxx
#include "sidl_BaseInterface.hxx"
#endif
#ifndef included_sidl_ClassInfo_hxx
#include "sidl_ClassInfo.hxx"
#endif
#ifndef included_sidl_NotImplementedException_hxx
#include "sidl_NotImplementedException.hxx"
#endif
// DO-NOT-DELETE splicer.begin(Inherit.H._includes)
// Put additional includes or other arbitrary code here...
// DO-NOT-DELETE splicer.end(Inherit.H._includes)

// default constructor, not to be used!
// user defined constructor
void Inherit::H_impl::_ctor() {
  // DO-NOT-DELETE splicer.begin(Inherit.H._ctor)
  // add construction details here
  // DO-NOT-DELETE splicer.end(Inherit.H._ctor)
}

// user defined destructor
void Inherit::H_impl::_dtor() {
  // DO-NOT-DELETE splicer.begin(Inherit.H._dtor)
  // add destruction details here
  // DO-NOT-DELETE splicer.end(Inherit.H._dtor)
}

// static class initializer
void Inherit::H_impl::_load() {
  // DO-NOT-DELETE splicer.begin(Inherit.H._load)
  // guaranteed to be called at most once before any other method in this class
  // DO-NOT-DELETE splicer.end(Inherit.H._load)
}

// user defined static methods: (none)

// user defined non-static methods:

// DO-NOT-DELETE splicer.begin(Inherit.H._misc)
// Put miscellaneous code here
// DO-NOT-DELETE splicer.end(Inherit.H._misc)

