#define sidl_string__array_create2dCol_m ary_create2dColr9crsbnpkm16jv_m
#define sidl_string__array_create2dcol_m ary_create2dcolr9crsbnpkm16jv_m
#define SIDL_STRING__ARRAY_CREATE2DCOL_M ARY_CREATE2DCOLR9CRSBNPKM16JV_M

#define sidl_string__array_create2dRow_m ary_create2dRowkjooq0k6ultghb_m
#define sidl_string__array_create2drow_m ary_create2drowkjooq0k6ultghb_m
#define SIDL_STRING__ARRAY_CREATE2DROW_M ARY_CREATE2DROWKJOOQ0K6ULTGHB_M

#define sidl_string__array_isColumnOrder_m ary_isColumnOrderzve2xzaq5ev9_m
#define sidl_string__array_iscolumnorder_m ary_iscolumnorderzve2xzaq5ev9_m
#define SIDL_STRING__ARRAY_ISCOLUMNORDER_M ARY_ISCOLUMNORDERZVE2XZAQ5EV9_M

