/*
 * File:          vDivByZeroExcept_Impl.java
 * Symbol:        vect.vDivByZeroExcept-v1.0
 * Symbol Type:   class
 * Babel Version: 1.4.0 (Revision: 6561M trunk)
 * Description:   Server-side implementation for vect.vDivByZeroExcept
 * 
 * WARNING: Automatically generated; only changes within splicers preserved
 * 
 */

package vect;

import sidl.BaseClass;
import sidl.BaseException;
import sidl.BaseInterface;
import sidl.ClassInfo;
import sidl.RuntimeException;
import sidl.SIDLException;
import sidl.io.Deserializer;
import sidl.io.Serializable;
import sidl.io.Serializer;
import vect.vExcept;

// DO-NOT-DELETE splicer.begin(vect.vDivByZeroExcept._imports)
// Nothing needed here.
// DO-NOT-DELETE splicer.end(vect.vDivByZeroExcept._imports)

/**
 * Symbol "vect.vDivByZeroExcept" (version 1.0)
 */
public class vDivByZeroExcept_Impl extends vDivByZeroExcept
{

  // DO-NOT-DELETE splicer.begin(vect.vDivByZeroExcept._data)
  // Nothing needed here.
  // DO-NOT-DELETE splicer.end(vect.vDivByZeroExcept._data)


  static { 
  // DO-NOT-DELETE splicer.begin(vect.vDivByZeroExcept._load)
  // Nothing needed here.
  // DO-NOT-DELETE splicer.end(vect.vDivByZeroExcept._load)

  }

  /**
   * User defined constructor
   */
  public vDivByZeroExcept_Impl(long IORpointer){
    super(IORpointer);
    // DO-NOT-DELETE splicer.begin(vect.vDivByZeroExcept.vDivByZeroExcept)
    return;
    // DO-NOT-DELETE splicer.end(vect.vDivByZeroExcept.vDivByZeroExcept)

  }

  /**
   * Back door constructor
   */
  public vDivByZeroExcept_Impl(){
    d_ior = _wrap(this);
    // DO-NOT-DELETE splicer.begin(vect.vDivByZeroExcept._wrap)
    return;
    // DO-NOT-DELETE splicer.end(vect.vDivByZeroExcept._wrap)

  }

  /**
   * User defined destructing method
   */
  public void dtor() throws Throwable{
    // DO-NOT-DELETE splicer.begin(vect.vDivByZeroExcept._dtor)
    return;
    // DO-NOT-DELETE splicer.end(vect.vDivByZeroExcept._dtor)

  }

  /**
   * finalize method (Only use this if you're sure you need it!)
   */
  public void finalize() throws Throwable{
    // DO-NOT-DELETE splicer.begin(vect.vDivByZeroExcept.finalize)
    return;
    // DO-NOT-DELETE splicer.end(vect.vDivByZeroExcept.finalize)

  }

  // user defined static methods: (none)

  // user defined non-static methods: (none)

  // DO-NOT-DELETE splicer.begin(vect.vDivByZeroExcept._misc)
  // Nothing needed here.
  // DO-NOT-DELETE splicer.end(vect.vDivByZeroExcept._misc)

} // end class vDivByZeroExcept

