/*
 * File:          synch.h
 * Symbol:        synch-v0.1
 * Symbol Type:   package
 * Babel Version: 1.4.0 (Revision: 6585 release-1-4-0-branch)
 * Description:   Client-side glue code for synch
 * 
 * WARNING: Automatically generated; changes will be lost
 * 
 */

#ifndef included_synch_RegOut_h
#include "synch_RegOut.h"
#endif
#ifndef included_synch_ResultType_h
#include "synch_ResultType.h"
#endif

