#define sidl_dcomplex__array_create2dCol_f ary_create2dCol3vzju1i5aj__jf_f
#define sidl_dcomplex__array_create2dcol_f ary_create2dcol3vzju1i5aj__jf_f
#define SIDL_DCOMPLEX__ARRAY_CREATE2DCOL_F ARY_CREATE2DCOL3VZJU1I5AJ__JF_F

#define sidl_dcomplex__array_create2dRow_f ary_create2dRowjoawaj2mvjnbfl_f
#define sidl_dcomplex__array_create2drow_f ary_create2drowjoawaj2mvjnbfl_f
#define SIDL_DCOMPLEX__ARRAY_CREATE2DROW_F ARY_CREATE2DROWJOAWAJ2MVJNBFL_F

#define sidl_dcomplex__array_createCol_f dc_ary_createColyr97ptn4ah5jw_f
#define sidl_dcomplex__array_createcol_f dc_ary_createcolyr97ptn4ah5jw_f
#define SIDL_DCOMPLEX__ARRAY_CREATECOL_F DC_ARY_CREATECOLYR97PTN4AH5JW_F

#define sidl_dcomplex__array_createRow_f dc_ary_createRowq0z2zuygy7f5x_f
#define sidl_dcomplex__array_createrow_f dc_ary_createrowq0z2zuygy7f5x_f
#define SIDL_DCOMPLEX__ARRAY_CREATEROW_F DC_ARY_CREATEROWQ0Z2ZUYGY7F5X_F

#define sidl_dcomplex__array_deleteRef_f dc_ary_deleteRefowjown5d_mkpr_f
#define sidl_dcomplex__array_deleteref_f dc_ary_deleterefowjown5d_mkpr_f
#define SIDL_DCOMPLEX__ARRAY_DELETEREF_F DC_ARY_DELETEREFOWJOWN5D_MKPR_F

#define sidl_dcomplex__array_isColumnOrder_f ary_isColumnOrder2idm90rtw92e_f
#define sidl_dcomplex__array_iscolumnorder_f ary_iscolumnorder2idm90rtw92e_f
#define SIDL_DCOMPLEX__ARRAY_ISCOLUMNORDER_F ARY_ISCOLUMNORDER2IDM90RTW92E_F

#define sidl_dcomplex__array_isRowOrder_f d_ary_isRowOrder7wlajzo7wds_t_f
#define sidl_dcomplex__array_isroworder_f d_ary_isroworder7wlajzo7wds_t_f
#define SIDL_DCOMPLEX__ARRAY_ISROWORDER_F D_ARY_ISROWORDER7WLAJZO7WDS_T_F

#define sidl_dcomplex__array_smartCopy_f dc_ary_smartCopypo1o3bi9a15pg_f
#define sidl_dcomplex__array_smartcopy_f dc_ary_smartcopypo1o3bi9a15pg_f
#define SIDL_DCOMPLEX__ARRAY_SMARTCOPY_F DC_ARY_SMARTCOPYPO1O3BI9A15PG_F

