#define sidl_string__array_create2dCol_f ary_create2dColt4s3pxby44m8zq_f
#define sidl_string__array_create2dcol_f ary_create2dcolt4s3pxby44m8zq_f
#define SIDL_STRING__ARRAY_CREATE2DCOL_F ARY_CREATE2DCOLT4S3PXBY44M8ZQ_F

#define sidl_string__array_create2dRow_f ary_create2dRowqty4s56rdsl6rw_f
#define sidl_string__array_create2drow_f ary_create2drowqty4s56rdsl6rw_f
#define SIDL_STRING__ARRAY_CREATE2DROW_F ARY_CREATE2DROWQTY4S56RDSL6RW_F

#define sidl_string__array_isColumnOrder_f ary_isColumnOrderu6_9xu_mqyfv_f
#define sidl_string__array_iscolumnorder_f ary_iscolumnorderu6_9xu_mqyfv_f
#define SIDL_STRING__ARRAY_ISCOLUMNORDER_F ARY_ISCOLUMNORDERU6_9XU_MQYFV_F

