#
# File:          CallType.py
# Symbol:        sidlx.rmi.CallType-v0.1
# Symbol Type:   enumeration
# Babel Version: 1.4.0 (Revision: 6607 release-1-4-0-branch)
# Description:   define enumeration constants
# 
# WARNING: Automatically generated; changes will be lost
#


"""There are 3 basic types of calls on the server side, CREATE, EXEC, and     \
  SERIAL.
This enumeration defines them.  
"""

CREATE = 0

EXEC = 1

SERIAL = 2

ERROR = 3

