#define sidl_fcomplex__array_create2dCol_m ary_create2dColjv3a2l5y8mzbo1_m
#define sidl_fcomplex__array_create2dcol_m ary_create2dcoljv3a2l5y8mzbo1_m
#define SIDL_FCOMPLEX__ARRAY_CREATE2DCOL_M ARY_CREATE2DCOLJV3A2L5Y8MZBO1_M

#define sidl_fcomplex__array_create2dRow_m ary_create2dRowo6kzd3bx6udmae_m
#define sidl_fcomplex__array_create2drow_m ary_create2drowo6kzd3bx6udmae_m
#define SIDL_FCOMPLEX__ARRAY_CREATE2DROW_M ARY_CREATE2DROWO6KZD3BX6UDMAE_M

#define sidl_fcomplex__array_createCol_m fc_ary_createColx0nc4ahh7fyui_m
#define sidl_fcomplex__array_createcol_m fc_ary_createcolx0nc4ahh7fyui_m
#define SIDL_FCOMPLEX__ARRAY_CREATECOL_M FC_ARY_CREATECOLX0NC4AHH7FYUI_M

#define sidl_fcomplex__array_createRow_m fc_ary_createRowzctgklf5zv5qj_m
#define sidl_fcomplex__array_createrow_m fc_ary_createrowzctgklf5zv5qj_m
#define SIDL_FCOMPLEX__ARRAY_CREATEROW_M FC_ARY_CREATEROWZCTGKLF5ZV5QJ_M

#define sidl_fcomplex__array_deleteRef_m fc_ary_deleteRef3qpuu6aysek9n_m
#define sidl_fcomplex__array_deleteref_m fc_ary_deleteref3qpuu6aysek9n_m
#define SIDL_FCOMPLEX__ARRAY_DELETEREF_M FC_ARY_DELETEREF3QPUU6AYSEK9N_M

#define sidl_fcomplex__array_isColumnOrder_m ary_isColumnOrder2nhheax67by__m
#define sidl_fcomplex__array_iscolumnorder_m ary_iscolumnorder2nhheax67by__m
#define SIDL_FCOMPLEX__ARRAY_ISCOLUMNORDER_M ARY_ISCOLUMNORDER2NHHEAX67BY__M

#define sidl_fcomplex__array_isRowOrder_m f_ary_isRowOrderdjtu_j44igez8_m
#define sidl_fcomplex__array_isroworder_m f_ary_isroworderdjtu_j44igez8_m
#define SIDL_FCOMPLEX__ARRAY_ISROWORDER_M F_ARY_ISROWORDERDJTU_J44IGEZ8_M

#define sidl_fcomplex__array_smartCopy_m fc_ary_smartCopygzyi35gw8gyde_m
#define sidl_fcomplex__array_smartcopy_m fc_ary_smartcopygzyi35gw8gyde_m
#define SIDL_FCOMPLEX__ARRAY_SMARTCOPY_M FC_ARY_SMARTCOPYGZYI35GW8GYDE_M

