// 
// File:          vect_vNegValExcept_Impl.cxx
// Symbol:        vect.vNegValExcept-v1.0
// Symbol Type:   class
// Babel Version: 1.4.0 (Revision: 6585 release-1-4-0-branch)
// Description:   Server-side implementation for vect.vNegValExcept
// 
// WARNING: Automatically generated; only changes within splicers preserved
// 
// 
#include "vect_vNegValExcept_Impl.hxx"

// 
// Includes for all method dependencies.
// 
#ifndef included_sidl_BaseInterface_hxx
#include "sidl_BaseInterface.hxx"
#endif
#ifndef included_sidl_ClassInfo_hxx
#include "sidl_ClassInfo.hxx"
#endif
#ifndef included_sidl_io_Deserializer_hxx
#include "sidl_io_Deserializer.hxx"
#endif
#ifndef included_sidl_io_Serializer_hxx
#include "sidl_io_Serializer.hxx"
#endif
#ifndef included_sidl_NotImplementedException_hxx
#include "sidl_NotImplementedException.hxx"
#endif
// DO-NOT-DELETE splicer.begin(vect.vNegValExcept._includes)
// Nothing needed here.
// DO-NOT-DELETE splicer.end(vect.vNegValExcept._includes)

// special constructor, used for data wrapping(required).  Do not put code here unless you really know what you're doing!
vect::vNegValExcept_impl::vNegValExcept_impl() : StubBase(reinterpret_cast< 
  void*>(::vect::vNegValExcept::_wrapObj(reinterpret_cast< void*>(this))),
  false) , _wrapped(true){ 
  // DO-NOT-DELETE splicer.begin(vect.vNegValExcept._ctor2)
  // Nothing needed here.
  // DO-NOT-DELETE splicer.end(vect.vNegValExcept._ctor2)
}

// user defined constructor
void vect::vNegValExcept_impl::_ctor() {
  // DO-NOT-DELETE splicer.begin(vect.vNegValExcept._ctor)
  // Nothing needed here.
  // DO-NOT-DELETE splicer.end(vect.vNegValExcept._ctor)
}

// user defined destructor
void vect::vNegValExcept_impl::_dtor() {
  // DO-NOT-DELETE splicer.begin(vect.vNegValExcept._dtor)
  // Nothing needed here.
  // DO-NOT-DELETE splicer.end(vect.vNegValExcept._dtor)
}

// static class initializer
void vect::vNegValExcept_impl::_load() {
  // DO-NOT-DELETE splicer.begin(vect.vNegValExcept._load)
  // Nothing needed here.
  // DO-NOT-DELETE splicer.end(vect.vNegValExcept._load)
}

// user defined static methods: (none)

// user defined non-static methods: (none)

// DO-NOT-DELETE splicer.begin(vect.vNegValExcept._misc)
// Nothing needed here.
// DO-NOT-DELETE splicer.end(vect.vNegValExcept._misc)

