/* Generated By:JJTree: Do not edit this line. ./SIDLParserTreeConstants.java */

package gov.llnl.babel.parsers.sidl2;

public interface SIDLParserTreeConstants
{
  public int JJTSTART = 0;
  public int JJTVERSION = 1;
  public int JJTREQUIRE = 2;
  public int JJTIMPORT = 3;
  public int JJTPACKAGE = 4;
  public int JJTUSERTYPE = 5;
  public int JJTTYPEATTRS = 6;
  public int JJTTYPEATTR = 7;
  public int JJTENUM = 8;
  public int JJTENUMERATOR = 9;
  public int JJTSTRUCT = 10;
  public int JJTSTRUCTITEM = 11;
  public int JJTCLASS = 12;
  public int JJTINTERFACE = 13;
  public int JJTEXTENDSONE = 14;
  public int JJTEXTENDSLIST = 15;
  public int JJTIMPLEMENTSLIST = 16;
  public int JJTIMPLEMENTSALLLIST = 17;
  public int JJTMETHOD = 18;
  public int JJTMETHODATTRS = 19;
  public int JJTRETURNTYPE = 20;
  public int JJTMETHODATTR = 21;
  public int JJTMETHODNAME = 22;
  public int JJTSHORTNAME = 23;
  public int JJTEXCEPTCLAUSE = 24;
  public int JJTFROMCLAUSE = 25;
  public int JJTINVARIANTS = 26;
  public int JJTREQUIRES = 27;
  public int JJTENSURES = 28;
  public int JJTASSERTION = 29;
  public int JJTNAME = 30;
  public int JJTARGLIST = 31;
  public int JJTARG = 32;
  public int JJTARGATTRS = 33;
  public int JJTARGATTR = 34;
  public int JJTVOID = 35;
  public int JJTCUSTOMATTR = 36;
  public int JJTMODE = 37;
  public int JJTPRIMATIVETYPE = 38;
  public int JJTARRAY = 39;
  public int JJTSCALARTYPE = 40;
  public int JJTDIMENSION = 41;
  public int JJTORIENTATION = 42;
  public int JJTRARRAY = 43;
  public int JJTEXTENTS = 44;
  public int JJTEXTENT = 45;
  public int JJTADD = 46;
  public int JJTMULT = 47;
  public int JJTIMPLICATION = 48;
  public int JJTOR = 49;
  public int JJTAND = 50;
  public int JJTBITWISE = 51;
  public int JJTEQUALITY = 52;
  public int JJTGTLT = 53;
  public int JJTSHIFT = 54;
  public int JJTPOWER = 55;
  public int JJTUNARY = 56;
  public int JJTFUNC = 57;
  public int JJTFUNCARGS = 58;
  public int JJTSCOPEDID = 59;
  public int JJTEXTENSION = 60;
  public int JJTLITERAL = 61;
  public int JJTCOMPLEX = 62;
  public int JJTNUMBER = 63;
  public int JJTINTEGER = 64;


  public String[] jjtNodeName = {
    "Start",
    "Version",
    "Require",
    "Import",
    "Package",
    "UserType",
    "TypeAttrs",
    "TypeAttr",
    "Enum",
    "Enumerator",
    "Struct",
    "StructItem",
    "Class",
    "Interface",
    "ExtendsOne",
    "ExtendsList",
    "ImplementsList",
    "ImplementsAllList",
    "Method",
    "MethodAttrs",
    "ReturnType",
    "MethodAttr",
    "MethodName",
    "ShortName",
    "ExceptClause",
    "FromClause",
    "Invariants",
    "Requires",
    "Ensures",
    "Assertion",
    "Name",
    "ArgList",
    "Arg",
    "ArgAttrs",
    "ArgAttr",
    "void",
    "CustomAttr",
    "Mode",
    "PrimativeType",
    "Array",
    "ScalarType",
    "Dimension",
    "Orientation",
    "Rarray",
    "Extents",
    "Extent",
    "Add",
    "Mult",
    "Implication",
    "Or",
    "And",
    "Bitwise",
    "Equality",
    "GtLt",
    "Shift",
    "Power",
    "Unary",
    "Func",
    "FuncArgs",
    "ScopedID",
    "Extension",
    "Literal",
    "Complex",
    "Number",
    "Integer",
  };
}
