#define sidl_dcomplex__array_create2dCol_m ary_create2dCol2u6j_sgipgll2t_m
#define sidl_dcomplex__array_create2dcol_m ary_create2dcol2u6j_sgipgll2t_m
#define SIDL_DCOMPLEX__ARRAY_CREATE2DCOL_M ARY_CREATE2DCOL2U6J_SGIPGLL2T_M

#define sidl_dcomplex__array_create2dRow_m ary_create2dRow9sm4o_qouamm6x_m
#define sidl_dcomplex__array_create2drow_m ary_create2drow9sm4o_qouamm6x_m
#define SIDL_DCOMPLEX__ARRAY_CREATE2DROW_M ARY_CREATE2DROW9SM4O_QOUAMM6X_M

#define sidl_dcomplex__array_createCol_m dc_ary_createColx2ge1e37_ypjq_m
#define sidl_dcomplex__array_createcol_m dc_ary_createcolx2ge1e37_ypjq_m
#define SIDL_DCOMPLEX__ARRAY_CREATECOL_M DC_ARY_CREATECOLX2GE1E37_YPJQ_M

#define sidl_dcomplex__array_createRow_m dc_ary_createRowioexvdf9k0anl_m
#define sidl_dcomplex__array_createrow_m dc_ary_createrowioexvdf9k0anl_m
#define SIDL_DCOMPLEX__ARRAY_CREATEROW_M DC_ARY_CREATEROWIOEXVDF9K0ANL_M

#define sidl_dcomplex__array_deleteRef_m dc_ary_deleteRefztvn8oh8dyf6__m
#define sidl_dcomplex__array_deleteref_m dc_ary_deleterefztvn8oh8dyf6__m
#define SIDL_DCOMPLEX__ARRAY_DELETEREF_M DC_ARY_DELETEREFZTVN8OH8DYF6__M

#define sidl_dcomplex__array_isColumnOrder_m ary_isColumnOrder9zj732lmvflb_m
#define sidl_dcomplex__array_iscolumnorder_m ary_iscolumnorder9zj732lmvflb_m
#define SIDL_DCOMPLEX__ARRAY_ISCOLUMNORDER_M ARY_ISCOLUMNORDER9ZJ732LMVFLB_M

#define sidl_dcomplex__array_isRowOrder_m d_ary_isRowOrderzjsla9w0ytvx9_m
#define sidl_dcomplex__array_isroworder_m d_ary_isroworderzjsla9w0ytvx9_m
#define SIDL_DCOMPLEX__ARRAY_ISROWORDER_M D_ARY_ISROWORDERZJSLA9W0YTVX9_M

#define sidl_dcomplex__array_smartCopy_m dc_ary_smartCopya_2jyr2got417_m
#define sidl_dcomplex__array_smartcopy_m dc_ary_smartcopya_2jyr2got417_m
#define SIDL_DCOMPLEX__ARRAY_SMARTCOPY_M DC_ARY_SMARTCOPYA_2JYR2GOT417_M

