/*
 * File:          sidlx_IOR.h
 * Symbol:        sidlx-v0.1
 * Symbol Type:   package
 * Babel Version: 1.4.0 (Revision: 6607 release-1-4-0-branch)
 * Description:   Intermediate Object Representation for sidlx
 * 
 * WARNING: Automatically generated; changes will be lost
 * 
 */

#ifndef included_sidlx_IOR_h
#define included_sidlx_IOR_h

/*
 * Symbol "sidlx" (version 0.1)
 * 
 * This package contains experimental extensions to sidl
 * and should not be used unless willing to tolerate its
 * disappearance in a following release
 */

#ifndef included_sidlx_rmi_IOR_h
#include "sidlx_rmi_IOR.h"
#endif

#endif
