#define sidl_opaque__array_create2dCol_m ary_create2dCol14olgqufbw9x36_m
#define sidl_opaque__array_create2dcol_m ary_create2dcol14olgqufbw9x36_m
#define SIDL_OPAQUE__ARRAY_CREATE2DCOL_M ARY_CREATE2DCOL14OLGQUFBW9X36_M

#define sidl_opaque__array_create2dRow_m ary_create2dRowveszjmbpz_ozec_m
#define sidl_opaque__array_create2drow_m ary_create2drowveszjmbpz_ozec_m
#define SIDL_OPAQUE__ARRAY_CREATE2DROW_M ARY_CREATE2DROWVESZJMBPZ_OZEC_M

#define sidl_opaque__array_isColumnOrder_m ary_isColumnOrdercgppo768anc2_m
#define sidl_opaque__array_iscolumnorder_m ary_iscolumnordercgppo768anc2_m
#define SIDL_OPAQUE__ARRAY_ISCOLUMNORDER_M ARY_ISCOLUMNORDERCGPPO768ANC2_M

