/*
 * File:          cons_IOR.h
 * Symbol:        cons-v1.0
 * Symbol Type:   package
 * Babel Version: 1.4.0 (Revision: 6585 release-1-4-0-branch)
 * Description:   Intermediate Object Representation for cons
 * 
 * WARNING: Automatically generated; changes will be lost
 * 
 */

#ifndef included_cons_IOR_h
#define included_cons_IOR_h

/*
 * Symbol "cons" (version 1.0)
 */

#ifndef included_cons_A_IOR_h
#include "cons_A_IOR.h"
#endif
#ifndef included_cons_B_IOR_h
#include "cons_B_IOR.h"
#endif
#ifndef included_cons_C_IOR_h
#include "cons_C_IOR.h"
#endif
#ifndef included_cons_D_IOR_h
#include "cons_D_IOR.h"
#endif

#endif
