/* Generated By:JJTree: Do not edit this line. ./SIDLParserVisitor.java */

package gov.llnl.babel.parsers.sidl2;

public interface SIDLParserVisitor
{
  public Object visit(ParseTreeNode node, Object data);
}
