#define sidl_double__array_create2dCol_m ary_create2dColhew3efdwl1b9gl_m
#define sidl_double__array_create2dcol_m ary_create2dcolhew3efdwl1b9gl_m
#define SIDL_DOUBLE__ARRAY_CREATE2DCOL_M ARY_CREATE2DCOLHEW3EFDWL1B9GL_M

#define sidl_double__array_create2dRow_m ary_create2dRowjxgy8npw6u2i40_m
#define sidl_double__array_create2drow_m ary_create2drowjxgy8npw6u2i40_m
#define SIDL_DOUBLE__ARRAY_CREATE2DROW_M ARY_CREATE2DROWJXGY8NPW6U2I40_M

#define sidl_double__array_isColumnOrder_m ary_isColumnOrderao7t2j7s1jz3_m
#define sidl_double__array_iscolumnorder_m ary_iscolumnorderao7t2j7s1jz3_m
#define SIDL_DOUBLE__ARRAY_ISCOLUMNORDER_M ARY_ISCOLUMNORDERAO7T2J7S1JZ3_M

