#define sidl_opaque__array_create2dCol_f ary_create2dColen6_y2gsaufp02_f
#define sidl_opaque__array_create2dcol_f ary_create2dcolen6_y2gsaufp02_f
#define SIDL_OPAQUE__ARRAY_CREATE2DCOL_F ARY_CREATE2DCOLEN6_Y2GSAUFP02_F

#define sidl_opaque__array_create2dRow_f ary_create2dRowgifogyhvxmzso3_f
#define sidl_opaque__array_create2drow_f ary_create2drowgifogyhvxmzso3_f
#define SIDL_OPAQUE__ARRAY_CREATE2DROW_F ARY_CREATE2DROWGIFOGYHVXMZSO3_F

#define sidl_opaque__array_isColumnOrder_f ary_isColumnOrder7vqgsucbx9vq_f
#define sidl_opaque__array_iscolumnorder_f ary_iscolumnorder7vqgsucbx9vq_f
#define SIDL_OPAQUE__ARRAY_ISCOLUMNORDER_F ARY_ISCOLUMNORDER7VQGSUCBX9VQ_F

