/*
 * File:          synch_ResultType_IOR.h
 * Symbol:        synch.ResultType-v0.1
 * Symbol Type:   enumeration
 * Babel Version: 1.4.0 (Revision: 6585 release-1-4-0-branch)
 * Description:   Intermediate Object Representation for synch.ResultType
 * 
 * WARNING: Automatically generated; changes will be lost
 */

#ifndef included_synch_ResultType_IOR_h
#define included_synch_ResultType_IOR_h

#ifndef included_sidlType_h
#include "sidlType.h"
#endif
#ifdef __cplusplus
extern "C" {
#endif

/*
 * Symbol "synch.ResultType" (version 0.1)
 * 
 * For short cut result method.
 */


/* Opaque forward declaration of array struct */
struct synch_ResultType__array;

enum synch_ResultType__enum {
  synch_ResultType_PASS  = 0,

  synch_ResultType_FAIL  = 1,

  synch_ResultType_XFAIL = 2,

  synch_ResultType_XPASS = 3

};

#ifdef __cplusplus
}
#endif
#endif
