#define sidl_long__array_isColumnOrder_f ary_isColumnOrderyej0_83wdqh2_f
#define sidl_long__array_iscolumnorder_f ary_iscolumnorderyej0_83wdqh2_f
#define SIDL_LONG__ARRAY_ISCOLUMNORDER_F ARY_ISCOLUMNORDERYEJ0_83WDQH2_F

