#define sidl_fcomplex__array_create2dCol_f ary_create2dColhva5ltge51zvxq_f
#define sidl_fcomplex__array_create2dcol_f ary_create2dcolhva5ltge51zvxq_f
#define SIDL_FCOMPLEX__ARRAY_CREATE2DCOL_F ARY_CREATE2DCOLHVA5LTGE51ZVXQ_F

#define sidl_fcomplex__array_create2dRow_f ary_create2dRow_fabbt5p47pn0l_f
#define sidl_fcomplex__array_create2drow_f ary_create2drow_fabbt5p47pn0l_f
#define SIDL_FCOMPLEX__ARRAY_CREATE2DROW_F ARY_CREATE2DROW_FABBT5P47PN0L_F

#define sidl_fcomplex__array_createCol_f fc_ary_createColywna2ds0tj9ro_f
#define sidl_fcomplex__array_createcol_f fc_ary_createcolywna2ds0tj9ro_f
#define SIDL_FCOMPLEX__ARRAY_CREATECOL_F FC_ARY_CREATECOLYWNA2DS0TJ9RO_F

#define sidl_fcomplex__array_createRow_f fc_ary_createRowjvfwqct4zycyr_f
#define sidl_fcomplex__array_createrow_f fc_ary_createrowjvfwqct4zycyr_f
#define SIDL_FCOMPLEX__ARRAY_CREATEROW_F FC_ARY_CREATEROWJVFWQCT4ZYCYR_F

#define sidl_fcomplex__array_deleteRef_f fc_ary_deleteRef88hazxdq9nays_f
#define sidl_fcomplex__array_deleteref_f fc_ary_deleteref88hazxdq9nays_f
#define SIDL_FCOMPLEX__ARRAY_DELETEREF_F FC_ARY_DELETEREF88HAZXDQ9NAYS_F

#define sidl_fcomplex__array_isColumnOrder_f ary_isColumnOrderufjotsoxs856_f
#define sidl_fcomplex__array_iscolumnorder_f ary_iscolumnorderufjotsoxs856_f
#define SIDL_FCOMPLEX__ARRAY_ISCOLUMNORDER_F ARY_ISCOLUMNORDERUFJOTSOXS856_F

#define sidl_fcomplex__array_isRowOrder_f f_ary_isRowOrderbo_514l9af9n7_f
#define sidl_fcomplex__array_isroworder_f f_ary_isroworderbo_514l9af9n7_f
#define SIDL_FCOMPLEX__ARRAY_ISROWORDER_F F_ARY_ISROWORDERBO_514L9AF9N7_F

#define sidl_fcomplex__array_smartCopy_f fc_ary_smartCopyjuy5wia82ayzm_f
#define sidl_fcomplex__array_smartcopy_f fc_ary_smartcopyjuy5wia82ayzm_f
#define SIDL_FCOMPLEX__ARRAY_SMARTCOPY_F FC_ARY_SMARTCOPYJUY5WIA82AYZM_F

