/*
 * File:          sidl_IOR.h
 * Symbol:        sidl-v0.9.17
 * Symbol Type:   package
 * Babel Version: 1.4.0 (Revision: 6607 release-1-4-0-branch)
 * Release:       $Name:  $
 * Revision:      @(#) $Id: $
 * Description:   Intermediate Object Representation for sidl
 * 
 * Copyright (c) 2000-2002, Lawrence Livermore National Security, LLC.
 * Produced at the Lawrence Livermore National Laboratory.
 * Written by the Components Team <components@llnl.gov>
 * All rights reserved.
 * 
 * This file is part of Babel. For more information, see
 * http://www.llnl.gov/CASC/components/. Please read the COPYRIGHT file
 * for Our Notice and the LICENSE file for the GNU Lesser General Public
 * License.
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License (as published by
 * the Free Software Foundation) version 2.1 dated February 1999.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the IMPLIED WARRANTY OF
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the terms and
 * conditions of the GNU Lesser General Public License for more details.
 * 
 * You should have recieved a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * WARNING: Automatically generated; changes will be lost
 * 
 */

#ifndef included_sidl_IOR_h
#define included_sidl_IOR_h

/*
 * Symbol "sidl" (version 0.9.17)
 * 
 * The <code>sidl</code> package contains the fundamental type and interface
 * definitions for the <code>sidl</code> interface definition language.  It
 * defines common run-time libraries and common base classes and interfaces.
 * Every interface implicitly inherits from <code>sidl.BaseInterface</code>
 * and every class implicitly inherits from <code>sidl.BaseClass</code>.
 */

#ifndef included_sidl_BaseClass_IOR_h
#include "sidl_BaseClass_IOR.h"
#endif
#ifndef included_sidl_BaseException_IOR_h
#include "sidl_BaseException_IOR.h"
#endif
#ifndef included_sidl_BaseInterface_IOR_h
#include "sidl_BaseInterface_IOR.h"
#endif
#ifndef included_sidl_CastException_IOR_h
#include "sidl_CastException_IOR.h"
#endif
#ifndef included_sidl_ClassInfo_IOR_h
#include "sidl_ClassInfo_IOR.h"
#endif
#ifndef included_sidl_ClassInfoI_IOR_h
#include "sidl_ClassInfoI_IOR.h"
#endif
#ifndef included_sidl_ClauseType_IOR_h
#include "sidl_ClauseType_IOR.h"
#endif
#ifndef included_sidl_ContractClass_IOR_h
#include "sidl_ContractClass_IOR.h"
#endif
#ifndef included_sidl_DFinder_IOR_h
#include "sidl_DFinder_IOR.h"
#endif
#ifndef included_sidl_DLL_IOR_h
#include "sidl_DLL_IOR.h"
#endif
#ifndef included_sidl_EnfPolicy_IOR_h
#include "sidl_EnfPolicy_IOR.h"
#endif
#ifndef included_sidl_EnfTraceLevel_IOR_h
#include "sidl_EnfTraceLevel_IOR.h"
#endif
#ifndef included_sidl_EnforceFreq_IOR_h
#include "sidl_EnforceFreq_IOR.h"
#endif
#ifndef included_sidl_Finder_IOR_h
#include "sidl_Finder_IOR.h"
#endif
#ifndef included_sidl_InvViolation_IOR_h
#include "sidl_InvViolation_IOR.h"
#endif
#ifndef included_sidl_LangSpecificException_IOR_h
#include "sidl_LangSpecificException_IOR.h"
#endif
#ifndef included_sidl_Loader_IOR_h
#include "sidl_Loader_IOR.h"
#endif
#ifndef included_sidl_MemAllocException_IOR_h
#include "sidl_MemAllocException_IOR.h"
#endif
#ifndef included_sidl_NotImplementedException_IOR_h
#include "sidl_NotImplementedException_IOR.h"
#endif
#ifndef included_sidl_PostViolation_IOR_h
#include "sidl_PostViolation_IOR.h"
#endif
#ifndef included_sidl_PreViolation_IOR_h
#include "sidl_PreViolation_IOR.h"
#endif
#ifndef included_sidl_Resolve_IOR_h
#include "sidl_Resolve_IOR.h"
#endif
#ifndef included_sidl_RuntimeException_IOR_h
#include "sidl_RuntimeException_IOR.h"
#endif
#ifndef included_sidl_SIDLException_IOR_h
#include "sidl_SIDLException_IOR.h"
#endif
#ifndef included_sidl_Scope_IOR_h
#include "sidl_Scope_IOR.h"
#endif
#ifndef included_sidl_io_IOR_h
#include "sidl_io_IOR.h"
#endif
#ifndef included_sidl_rmi_IOR_h
#include "sidl_rmi_IOR.h"
#endif

#endif
