/*
 * File:          sidlx.h
 * Symbol:        sidlx-v0.1
 * Symbol Type:   package
 * Babel Version: 1.4.0 (Revision: 6607 release-1-4-0-branch)
 * Description:   Client-side glue code for sidlx
 * 
 * WARNING: Automatically generated; changes will be lost
 * 
 */

#ifndef included_sidlx_rmi_h
#include "sidlx_rmi.h"
#endif

