/*
 * File:          sidlx_rmi_IOR.h
 * Symbol:        sidlx.rmi-v0.1
 * Symbol Type:   package
 * Babel Version: 1.4.0 (Revision: 6607 release-1-4-0-branch)
 * Description:   Intermediate Object Representation for sidlx.rmi
 * 
 * WARNING: Automatically generated; changes will be lost
 * 
 */

#ifndef included_sidlx_rmi_IOR_h
#define included_sidlx_rmi_IOR_h

/*
 * Symbol "sidlx.rmi" (version 0.1)
 * 
 *  
 */

#ifndef included_sidlx_rmi_BadFileDescriptorException_IOR_h
#include "sidlx_rmi_BadFileDescriptorException_IOR.h"
#endif
#ifndef included_sidlx_rmi_CallType_IOR_h
#include "sidlx_rmi_CallType_IOR.h"
#endif
#ifndef included_sidlx_rmi_ChildSocket_IOR_h
#include "sidlx_rmi_ChildSocket_IOR.h"
#endif
#ifndef included_sidlx_rmi_ClientSocket_IOR_h
#include "sidlx_rmi_ClientSocket_IOR.h"
#endif
#ifndef included_sidlx_rmi_Common_IOR_h
#include "sidlx_rmi_Common_IOR.h"
#endif
#ifndef included_sidlx_rmi_ConnectionRefusedException_IOR_h
#include "sidlx_rmi_ConnectionRefusedException_IOR.h"
#endif
#ifndef included_sidlx_rmi_ConnectionResetException_IOR_h
#include "sidlx_rmi_ConnectionResetException_IOR.h"
#endif
#ifndef included_sidlx_rmi_GenNetworkException_IOR_h
#include "sidlx_rmi_GenNetworkException_IOR.h"
#endif
#ifndef included_sidlx_rmi_IPv4Socket_IOR_h
#include "sidlx_rmi_IPv4Socket_IOR.h"
#endif
#ifndef included_sidlx_rmi_JimEchoServer_IOR_h
#include "sidlx_rmi_JimEchoServer_IOR.h"
#endif
#ifndef included_sidlx_rmi_NetworkUnreachableException_IOR_h
#include "sidlx_rmi_NetworkUnreachableException_IOR.h"
#endif
#ifndef included_sidlx_rmi_NoServerException_IOR_h
#include "sidlx_rmi_NoServerException_IOR.h"
#endif
#ifndef included_sidlx_rmi_NotEnoughMemoryException_IOR_h
#include "sidlx_rmi_NotEnoughMemoryException_IOR.h"
#endif
#ifndef included_sidlx_rmi_OutOfAddressesException_IOR_h
#include "sidlx_rmi_OutOfAddressesException_IOR.h"
#endif
#ifndef included_sidlx_rmi_RecoverableException_IOR_h
#include "sidlx_rmi_RecoverableException_IOR.h"
#endif
#ifndef included_sidlx_rmi_RetryException_IOR_h
#include "sidlx_rmi_RetryException_IOR.h"
#endif
#ifndef included_sidlx_rmi_ServerSocket_IOR_h
#include "sidlx_rmi_ServerSocket_IOR.h"
#endif
#ifndef included_sidlx_rmi_Settings_IOR_h
#include "sidlx_rmi_Settings_IOR.h"
#endif
#ifndef included_sidlx_rmi_SimCall_IOR_h
#include "sidlx_rmi_SimCall_IOR.h"
#endif
#ifndef included_sidlx_rmi_SimHandle_IOR_h
#include "sidlx_rmi_SimHandle_IOR.h"
#endif
#ifndef included_sidlx_rmi_SimReturn_IOR_h
#include "sidlx_rmi_SimReturn_IOR.h"
#endif
#ifndef included_sidlx_rmi_SimpleOrb_IOR_h
#include "sidlx_rmi_SimpleOrb_IOR.h"
#endif
#ifndef included_sidlx_rmi_SimpleServer_IOR_h
#include "sidlx_rmi_SimpleServer_IOR.h"
#endif
#ifndef included_sidlx_rmi_SimpleTicket_IOR_h
#include "sidlx_rmi_SimpleTicket_IOR.h"
#endif
#ifndef included_sidlx_rmi_SimpleTicketBook_IOR_h
#include "sidlx_rmi_SimpleTicketBook_IOR.h"
#endif
#ifndef included_sidlx_rmi_Simsponse_IOR_h
#include "sidlx_rmi_Simsponse_IOR.h"
#endif
#ifndef included_sidlx_rmi_Simvocation_IOR_h
#include "sidlx_rmi_Simvocation_IOR.h"
#endif
#ifndef included_sidlx_rmi_Socket_IOR_h
#include "sidlx_rmi_Socket_IOR.h"
#endif
#ifndef included_sidlx_rmi_Statistics_IOR_h
#include "sidlx_rmi_Statistics_IOR.h"
#endif
#ifndef included_sidlx_rmi_TimeoutException_IOR_h
#include "sidlx_rmi_TimeoutException_IOR.h"
#endif
#ifndef included_sidlx_rmi_TooManyOpenFilesException_IOR_h
#include "sidlx_rmi_TooManyOpenFilesException_IOR.h"
#endif
#ifndef included_sidlx_rmi_UnauthorizedCallException_IOR_h
#include "sidlx_rmi_UnauthorizedCallException_IOR.h"
#endif
#ifndef included_sidlx_rmi_UnexpectedCloseException_IOR_h
#include "sidlx_rmi_UnexpectedCloseException_IOR.h"
#endif
#ifndef included_sidlx_rmi_UnrecognizedNetworkException_IOR_h
#include "sidlx_rmi_UnrecognizedNetworkException_IOR.h"
#endif
#ifndef included_sidlx_rmi_UnrecoverableException_IOR_h
#include "sidlx_rmi_UnrecoverableException_IOR.h"
#endif

#endif
