#
# File:          __init__.py
# Symbol:        sidlx.rmi-v0.1
# Symbol Type:   package
# Babel Version: 1.4.0 (Revision: 6607 release-1-4-0-branch)
# Description:   package initialization code
# 
# WARNING: Automatically generated; changes will be lost
# 
#


""" 
"""

__all__ = [
   "BadFileDescriptorException",
   "CallType",
   "ChildSocket",
   "ClientSocket",
   "Common",
   "ConnectionRefusedException",
   "ConnectionResetException",
   "GenNetworkException",
   "IPv4Socket",
   "JimEchoServer",
   "NetworkUnreachableException",
   "NoServerException",
   "NotEnoughMemoryException",
   "OutOfAddressesException",
   "RecoverableException",
   "RetryException",
   "ServerSocket",
   "Settings",
   "SimCall",
   "SimHandle",
   "SimReturn",
   "SimpleOrb",
   "SimpleServer",
   "SimpleTicket",
   "SimpleTicketBook",
   "Simsponse",
   "Simvocation",
   "Socket",
   "Statistics",
   "TimeoutException",
   "TooManyOpenFilesException",
   "UnauthorizedCallException",
   "UnexpectedCloseException",
   "UnrecognizedNetworkException",
   "UnrecoverableException" ]

try:
  from pkgutil import extend_path
  __path__ = extend_path(__path__, __name__)
except: # ignore all exceptions
  pass
