/*
 * File:          synch_IOR.h
 * Symbol:        synch-v0.1
 * Symbol Type:   package
 * Babel Version: 1.4.0 (Revision: 6585 release-1-4-0-branch)
 * Description:   Intermediate Object Representation for synch
 * 
 * WARNING: Automatically generated; changes will be lost
 * 
 */

#ifndef included_synch_IOR_h
#define included_synch_IOR_h

/*
 * Symbol "synch" (version 0.1)
 * 
 * Provide a simple library to write to standard output or standard error.
 * This can be used to synchronize output from multiple languages.
 */

#ifndef included_synch_RegOut_IOR_h
#include "synch_RegOut_IOR.h"
#endif
#ifndef included_synch_ResultType_IOR_h
#include "synch_ResultType_IOR.h"
#endif

#endif
