#define sidl_double__array_create2dCol_f ary_create2dColrj0gb1djcbkrhs_f
#define sidl_double__array_create2dcol_f ary_create2dcolrj0gb1djcbkrhs_f
#define SIDL_DOUBLE__ARRAY_CREATE2DCOL_F ARY_CREATE2DCOLRJ0GB1DJCBKRHS_F

#define sidl_double__array_create2dRow_f ary_create2dRowunx6e0f9ajlsbl_f
#define sidl_double__array_create2drow_f ary_create2drowunx6e0f9ajlsbl_f
#define SIDL_DOUBLE__ARRAY_CREATE2DROW_F ARY_CREATE2DROWUNX6E0F9AJLSBL_F

#define sidl_double__array_isColumnOrder_f ary_isColumnOrderb43602s_rbxo_f
#define sidl_double__array_iscolumnorder_f ary_iscolumnorderb43602s_rbxo_f
#define SIDL_DOUBLE__ARRAY_ISCOLUMNORDER_F ARY_ISCOLUMNORDERB43602S_RBXO_F

