#define sidl_char__array_isColumnOrder_f ary_isColumnOrderqj4bo7rpnc6k_f
#define sidl_char__array_iscolumnorder_f ary_iscolumnorderqj4bo7rpnc6k_f
#define SIDL_CHAR__ARRAY_ISCOLUMNORDER_F ARY_ISCOLUMNORDERQJ4BO7RPNC6K_F

