/*
 * File:          sidlx_rmi.h
 * Symbol:        sidlx.rmi-v0.1
 * Symbol Type:   package
 * Babel Version: 1.4.0 (Revision: 6607 release-1-4-0-branch)
 * Description:   Client-side glue code for sidlx.rmi
 * 
 * WARNING: Automatically generated; changes will be lost
 * 
 */

#ifndef included_sidlx_rmi_BadFileDescriptorException_h
#include "sidlx_rmi_BadFileDescriptorException.h"
#endif
#ifndef included_sidlx_rmi_CallType_h
#include "sidlx_rmi_CallType.h"
#endif
#ifndef included_sidlx_rmi_ChildSocket_h
#include "sidlx_rmi_ChildSocket.h"
#endif
#ifndef included_sidlx_rmi_ClientSocket_h
#include "sidlx_rmi_ClientSocket.h"
#endif
#ifndef included_sidlx_rmi_Common_h
#include "sidlx_rmi_Common.h"
#endif
#ifndef included_sidlx_rmi_ConnectionRefusedException_h
#include "sidlx_rmi_ConnectionRefusedException.h"
#endif
#ifndef included_sidlx_rmi_ConnectionResetException_h
#include "sidlx_rmi_ConnectionResetException.h"
#endif
#ifndef included_sidlx_rmi_GenNetworkException_h
#include "sidlx_rmi_GenNetworkException.h"
#endif
#ifndef included_sidlx_rmi_IPv4Socket_h
#include "sidlx_rmi_IPv4Socket.h"
#endif
#ifndef included_sidlx_rmi_JimEchoServer_h
#include "sidlx_rmi_JimEchoServer.h"
#endif
#ifndef included_sidlx_rmi_NetworkUnreachableException_h
#include "sidlx_rmi_NetworkUnreachableException.h"
#endif
#ifndef included_sidlx_rmi_NoServerException_h
#include "sidlx_rmi_NoServerException.h"
#endif
#ifndef included_sidlx_rmi_NotEnoughMemoryException_h
#include "sidlx_rmi_NotEnoughMemoryException.h"
#endif
#ifndef included_sidlx_rmi_OutOfAddressesException_h
#include "sidlx_rmi_OutOfAddressesException.h"
#endif
#ifndef included_sidlx_rmi_RecoverableException_h
#include "sidlx_rmi_RecoverableException.h"
#endif
#ifndef included_sidlx_rmi_RetryException_h
#include "sidlx_rmi_RetryException.h"
#endif
#ifndef included_sidlx_rmi_ServerSocket_h
#include "sidlx_rmi_ServerSocket.h"
#endif
#ifndef included_sidlx_rmi_Settings_h
#include "sidlx_rmi_Settings.h"
#endif
#ifndef included_sidlx_rmi_SimCall_h
#include "sidlx_rmi_SimCall.h"
#endif
#ifndef included_sidlx_rmi_SimHandle_h
#include "sidlx_rmi_SimHandle.h"
#endif
#ifndef included_sidlx_rmi_SimReturn_h
#include "sidlx_rmi_SimReturn.h"
#endif
#ifndef included_sidlx_rmi_SimpleOrb_h
#include "sidlx_rmi_SimpleOrb.h"
#endif
#ifndef included_sidlx_rmi_SimpleServer_h
#include "sidlx_rmi_SimpleServer.h"
#endif
#ifndef included_sidlx_rmi_SimpleTicket_h
#include "sidlx_rmi_SimpleTicket.h"
#endif
#ifndef included_sidlx_rmi_SimpleTicketBook_h
#include "sidlx_rmi_SimpleTicketBook.h"
#endif
#ifndef included_sidlx_rmi_Simsponse_h
#include "sidlx_rmi_Simsponse.h"
#endif
#ifndef included_sidlx_rmi_Simvocation_h
#include "sidlx_rmi_Simvocation.h"
#endif
#ifndef included_sidlx_rmi_Socket_h
#include "sidlx_rmi_Socket.h"
#endif
#ifndef included_sidlx_rmi_Statistics_h
#include "sidlx_rmi_Statistics.h"
#endif
#ifndef included_sidlx_rmi_TimeoutException_h
#include "sidlx_rmi_TimeoutException.h"
#endif
#ifndef included_sidlx_rmi_TooManyOpenFilesException_h
#include "sidlx_rmi_TooManyOpenFilesException.h"
#endif
#ifndef included_sidlx_rmi_UnauthorizedCallException_h
#include "sidlx_rmi_UnauthorizedCallException.h"
#endif
#ifndef included_sidlx_rmi_UnexpectedCloseException_h
#include "sidlx_rmi_UnexpectedCloseException.h"
#endif
#ifndef included_sidlx_rmi_UnrecognizedNetworkException_h
#include "sidlx_rmi_UnrecognizedNetworkException.h"
#endif
#ifndef included_sidlx_rmi_UnrecoverableException_h
#include "sidlx_rmi_UnrecoverableException.h"
#endif

