package gov.llnl.babel.ast;

import gov.llnl.babel.parsers.sidl2.ParseTreeNode;

public class NumericLiteral extends Literal {

  public NumericLiteral(ParseTreeNode src, ASTNode parent) {
    super(src, parent);
    // TODO Auto-generated constructor stub
  }

}
